# -*- coding: utf-8 -*-

"""livereload.task

Task management for LiveReload Server.

A basic syntax overview::

    from livereload.task import Task

    Task.add('file.css')

    def do_some_thing():
        pass

    Task.add('file.css', do_some_thing)
"""

import os
import glob

IGNORE = [
    '.pyc', '.pyo', '.o',
]


class Task(object):
    tasks = {}
    _modified_times = {}

    @classmethod
    def add(cls, path, func=None):
        cls.tasks[path] = func

    @classmethod
    def watch(cls):
        result = False
        changes = []
        for path in cls.tasks:
            if cls._checking(path):
                result = True
                changes.append(path)
                func = cls.tasks[path]
                if func:
                    func()

        return result and changes

    @classmethod
    def _check_file(cls, path):
        for ext in IGNORE:
            if path.endswith(ext):
                return False

        if not os.path.isfile(path):
            return False

        modified = os.stat(path).st_mtime

        if path not in cls._modified_times:
            cls._modified_times[path] = modified
            return False

        if path in cls._modified_times and \
           cls._modified_times[path] == modified:
            return False

        return True

    @classmethod
    def _checking(cls, path):
        if os.path.isfile(path):
            return cls._check_file(path)

        if os.path.isdir(path):
            result = False
            for root, dirs, files in os.walk(path):
                #: don't watch version control dirs
                if '.git' in dirs:
                    dirs.remove('.git')
                if '.hg' in dirs:
                    dirs.remove('.hg')
                if '.svn' in dirs:
                    dirs.remove('.svn')
                for f in files:
                    path = os.path.join(root, f)
                    result = cls._check_file(path) or result

            return result

        result = False
        for f in glob.glob(path):
            result = cls._check_file(f) or result

        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
ROOT = os.path.dirname(__file__)

import sys
kwargs = {}
kwargs['include_package_data'] = True
major, minor = sys.version_info[:2]
if major >= 3:
    kwargs['use_2to3'] = True

from setuptools import setup, find_packages
import livereload
from email.utils import parseaddr
author, author_email = parseaddr(livereload.__author__)

setup(
    name='livereload',
    version=livereload.__version__,
    author=author,
    author_email=author_email,
    url=livereload.__homepage__,
    packages=find_packages(),
    description='Python LiveReload is an awesome tool for web developers',
    long_description=livereload.__doc__,
    entry_points={
        'console_scripts': ['livereload= livereload.app:main'],
    },
    install_requires=[
        'tornado',
    ],
    license=open(os.path.join(ROOT, 'LICENSE')).read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment :: Mozilla',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Compilers',
        'Topic :: Software Development :: Debuggers',
    ],
    **kwargs
)



# -*- coding: utf-8 -*-

"""livereload.app

Core Server of LiveReload.
"""

import os
import sys
import logging
import tornado.web
import tornado.options
import tornado.ioloop
from tornado import escape
from tornado import websocket
from tornado.util import ObjectDict
from livereload.task import Task


ROOT = os.path.abspath(os.path.dirname(__file__))
STATIC_PATH = os.path.join(ROOT, 'livereload-js/dist')

NOTIFIER = None
APPLICATION_ICON = None


def _get_growl():
    import gntp.notifier
    growl = gntp.notifier.GrowlNotifier(
        applicationName='Python LiveReload',
        notifications=['Message'],
        defaultNotifications=['Message'],
        applicationIcon=APPLICATION_ICON,
    )
    result = growl.register()
    if result is not True:
        return None

    def notifier(message):
        return growl.notify(
            'Message',
            'LiveReload',
            message,
            icon=APPLICATION_ICON,
        )

    return notifier


def _get_notifyOSD():
    import pynotify
    pynotify.init('Python LiveReload')
    return lambda message: pynotify.Notification('LiveReload', message).show()


def send_notify(message):
    global NOTIFIER
    if NOTIFIER:
        return NOTIFIER(message)
    try:
        NOTIFIER = _get_growl()
    except:
        try:
            NOTIFIER = _get_notifyOSD()
        except:
            NOTIFIER = logging.info

    return NOTIFIER(message)


class LiveReloadHandler(websocket.WebSocketHandler):
    waiters = set()
    _watch_running = False

    def allow_draft76(self):
        return True

    def on_close(self):
        if self in LiveReloadHandler.waiters:
            LiveReloadHandler.waiters.remove(self)
            #send_notify('There are %s waiters left' % len(self.waiters))

    def send_message(self, message):
        if isinstance(message, dict):
            message = escape.json_encode(message)

        try:
            self.write_message(message)
        except:
            logging.error('Error sending message', exc_info=True)

    def watch_tasks(self):

        changes = Task.watch()
        if changes:
            send_notify(
                'Reload %s waiters'
                '\nChanged %s'
                % (len(LiveReloadHandler.waiters), changes)
            )
            msg = {
                'command': 'reload',
                'path': '*',
                'liveCSS': True
            }
            for waiter in LiveReloadHandler.waiters:
                try:
                    waiter.write_message(msg)
                except:
                    logging.error('Error sending message', exc_info=True)
                    LiveReloadHandler.waiters.remove(waiter)

    def on_message(self, message):
        """Handshake with livereload.js

        1. client send 'hello'
        2. server reply 'hello'
        3. client send 'info'

        http://help.livereload.com/kb/ecosystem/livereload-protocol
        """
        message = ObjectDict(escape.json_decode(message))
        if message.command == 'hello':
            handshake = {}
            handshake['command'] = 'hello'
            protocols = message.protocols
            protocols.append(
                'http://livereload.com/protocols/2.x-remote-control'
            )
            handshake['protocols'] = protocols
            handshake['serverName'] = 'livereload-tornado'
            self.send_message(handshake)

        if message.command == 'info' and 'url' in message:
            send_notify('Browser Connected: %s' % message.url)
            LiveReloadHandler.waiters.add(self)
            if not LiveReloadHandler._watch_running:
                try:
                    execfile('Guardfile')
                except:
                    Task.add(os.getcwd())

                LiveReloadHandler._watch_running = True
                logging.info('Start watching changes')
                tornado.ioloop.PeriodicCallback(self.watch_tasks, 500).start()


class IndexHandler(tornado.web.StaticFileHandler):
    def get(self, path=None):
        if not path:
            super(IndexHandler, self).get('index.html')
            return

        if path.endswith('/'):
            root = path

            path = path + 'index.html'
            if not os.path.exists(path):
                self.create_index(root)
                return

        super(IndexHandler, self).get(path)

    def create_index(self, root):
        files = os.listdir(root)
        self.write('<ul>')
        for f in files:
            path = os.path.join(root, f)
            self.write('<li>')
            if os.path.isdir(path):
                self.write('<a href="%s/">%s</a>' % (f, f))
            else:
                self.write('<a href="%s">%s</a>' % (f, f))
            self.write('</li>')

        self.write('</ul>')


handlers = [
    (r'/livereload', LiveReloadHandler),
    (
        r'/(livereload.js)',
        tornado.web.StaticFileHandler,
        {'path': STATIC_PATH}
    ),
    (r'/(.*)', IndexHandler, {'path': '.'}),
    (r'/', IndexHandler, {'path': '.'}),
]


def main():
    if len(sys.argv) > 1:
        #: command-line tools like Makefile
        execfile('Guardfile')
        for cmd in sys.argv[1:]:
            print(cmd)
            exec('%s()' % cmd)
        return

    #: option config is not available
    #: but this enables pretty colorful logging
    tornado.options.parse_command_line()
    app = tornado.web.Application(handlers=handlers)
    app.listen(35729)
    #: task watch initialize
    Task.watch()
    print('Start service at  127.0.0.1:35729')
    tornado.ioloop.IOLoop.instance().start()

if __name__ == '__main__':
    main()



#!/usr/bin/python

"""livereload.compiler

Provides a set of compilers for web developers.

Available compilers now:

+ less
+ uglifyjs
+ slimmer
"""

import os
import functools
from subprocess import Popen, PIPE


def make_folder(dest):
    folder = os.path.split(dest)[0]
    if not folder:
        return
    if os.path.isdir(folder):
        return
    try:
        os.makedirs(folder)
    except:
        pass


def _get_http_file(url, build_dir='build/assets'):
    import hashlib
    key = hashlib.md5(url).hexdigest()
    filename = os.path.join(os.getcwd(), build_dir, key)
    if os.path.exists(filename):
        return filename
    make_folder(filename)

    import urllib
    print('Downloading: %s' % url)
    urllib.urlretrieve(url, filename)
    return filename


class BaseCompiler(object):
    """BaseCompiler

    BaseCompiler defines the basic syntax of a Compiler.

    >>> c = BaseCompiler('a')
    >>> c.write('b')  #: write compiled code to 'b'
    >>> c.append('c')  #: append compiled code to 'c'
    """
    def __init__(self, path):
        self.filetype = os.path.splitext(path)[1]

        if path.startswith('http://') or path.startswith('https://'):
            path = _get_http_file(path)

        self.path = path

    def _get_code(self):
        f = open(self.path)
        code = f.read()
        f.close()
        return code

    def write(self, output):
        """write code to output"""
        make_folder(output)
        f = open(output, 'w')
        f.write(self._get_code())
        f.close()

    def append(self, output):
        """append code to output"""
        make_folder(output)
        f = open(output, 'a')
        f.write(self._get_code())
        f.close()


class _CommandCompiler(BaseCompiler):
    command = ''
    command_options = ''

    def _get_code(self):
        cmd = [self.command, self.command_options, self.path]
        p = Popen(cmd, stdout=PIPE, stderr=PIPE)
        stdout, stderr = p.communicate()
        if stderr:
            raise Exception(stderr)
        #: stdout is bytes, decode for python3
        return stdout.decode()


class LessCompiler(_CommandCompiler):
    command = 'lessc'
    command_options = '--compress'


def lessc(path, output, mode='w'):
    def _compile(path, output, mode):
        c = LessCompiler(path)
        if mode == 'a':
            c.append(output)
            return
        c.write(output)
        return
    return functools.partial(_compile, path, output, mode)


class UglifyJSCompiler(_CommandCompiler):
    command = 'uglifyjs'
    command_options = '-nc'


def uglifyjs(path, output, mode='w'):
    def _compile(path, output, mode):
        c = UglifyJSCompiler(path)
        if mode == 'a':
            c.append(output)
            return
        c.write(output)
        return
    return functools.partial(_compile, path, output, mode)


class SlimmerCompiler(BaseCompiler):
    def _get_code(self):
        import slimmer
        f = open(self.path)
        code = f.read()
        f.close()
        if self.filetype == '.css':
            return slimmer.css_slimmer(code)
        if self.filetype == '.js':
            return slimmer.js_slimmer(code)
        if self.filetype == '.html':
            return slimmer.xhtml_slimmer(code)
        return code


def slimmer(path, output, mode='w'):
    def _compile(path, output, mode):
        c = SlimmerCompiler(path)
        if mode == 'a':
            c.append(output)
            return
        c.write(output)
        return
    return functools.partial(_compile, path, output, mode)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2012, Hsiaoming Yang <http://lepture.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#    * Neither the name of the author nor the names of its contributors
#      may be used to endorse or promote products derived from this
#      software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Python LiveReload
=================

`LiveReload  <http://livereload.com/>`_ Server in Python Version.

Web Developers need to refresh a browser everytime when he saved a file (css,
javascript, html), it is really boring. LiveReload will take care of that for
you. When you saved a file, your browser will refresh itself. And what's more,
it can do some tasks like compiling less to css before the browser refreshing.

Installation
------------

Python LiveReload is designed for web developers who know Python.

Install python-livereload
~~~~~~~~~~~~~~~~~~~~~~~~~

Install Python LiveReload with pip::

    $ pip install livereload

If you don't have pip installed, try easy_install::

    $ easy_install livereload


Install Browser Extensions
~~~~~~~~~~~~~~~~~~~~~~~~~~

Get Browser Extensions From LiveReload.com

+ Chrome Extension
+ Safari Extension
+ Firefox Extension

Visit: http://help.livereload.com/kb/general-use/browser-extensions

Get Notification
~~~~~~~~~~~~~~~~~

If you are on Mac, and you are a Growl user::

    $ pip install gntp

If you are on Ubuntu, you don't need to do anything. Notification just works.

Working with file protocal
~~~~~~~~~~~~~~~~~~~~~~~~~~

Enable file protocal on Chrome:

.. image:: http://i.imgur.com/qGpJI.png


Quickstart
------------

LiveReload is designed for more complex tasks, not just for refreshing a
browser. But you can still do the simple task.

Assume you have livereload and its extension installed, and now you are in your
working directory. With command::

    $ livereload

your browser will reload, if any file in the working directory changed.


Guardfile
----------
More complex tasks can be done by Guardfile. Write a Guardfile in your working
directory, the basic syntax::

    #!/usr/bin/env python
    from livereload.task import Task

    Task.add('static/style.css')
    Task.add('*.html')

Now livereload will only guard static/style.css and html in your workding
directory.

But python-livereload is more than that, you can specify a task before
refreshing the browser::

    #!/usr/bin/env python
    from livereload.task import Task
    from livereload.compiler import lessc

    Task.add('style.less', lessc('style.less', 'style.css'))

And it will compile less css before refreshing the browser now.


Others
--------

If you are on a Mac, you can buy `LiveReload2 <http://livereload.com/>`_.

If you are a rubist, you can get guard-livereload.
"""

__version__ = '0.8'
__author__ = 'Hsiaoming Yang <lepture@me.com>'
__homepage__ = 'http://lepture.com/project/livereload/'
